import { Comparator } from "@rickosborne/typical";
/**
 * Mixin for a comparator with chaining functions to constrain
 * the behavior of the comparator.
 */
export type ChainableComparator<T> = Comparator<T> & {
    /**
     * Reverse the comparator.
     */
    get desc(): ChainableComparator<T>;
    /**
     * Return null and undefined values before the rest.
     */
    get nullsFirst(): ChainableComparator<T | undefined | null>;
    /**
     * Return null and undefined values after the rest.
     */
    get nullsLast(): ChainableComparator<T | undefined | null>;
};
/**
 * Wrap the given comparator to return null and undefined values
 * before the others.
 */
export declare const withNullsFirst: <T>(comparator: Comparator<T>) => Comparator<T | null | undefined>;
/**
 * Wrap the given comparator to return null and undefined values
 * after the others.
 */
export declare const withNullsLast: <T>(comparator: Comparator<T>) => Comparator<T | null | undefined>;
/**
 * Wrap the given comparator so you can chain {@link ChainableComparator}
 * functions from it.
 */
export declare const chainableComparator: <T>(comparator: Comparator<T>) => ChainableComparator<T>;
/**
 * Numeric comparator which sorts lower values before higher ones.
 */
export declare const numberAsc: Comparator<number>;
/**
 * Build a comparator for the given type, which goes through the
 * specified checks in the order they are defined.
 */
export type ComparatorBuilder<T> = {
    /**
     * Return the generated comparator.
     */
    build(): Comparator<T>;
    /**
     * Invert the previous comparator.
     */
    get desc(): ComparatorBuilder<T>;
    /**
     * Have the previous comparator return null and undefined values before others.
     */
    get nullsFirst(): ComparatorBuilder<T>;
    /**
     * Have the previous comparator return null and undefined values after others.
     */
    get nullsLast(): ComparatorBuilder<T>;
    /**
     * Add comparator which first accesses a numeric value, often
     * a property value, from the compared type.
     */
    num(accessor: (t: T) => (undefined | number)): ComparatorBuilder<T>;
    /**
     * Add comparator which first accesses a string value, often
     * a property value, from the compared type.
     */
    str(accessor: (t: T) => (undefined | string)): ComparatorBuilder<T>;
};
/**
 * Start building a comparator.
 */
export declare const comparatorBuilder: <T>() => ComparatorBuilder<T>;
//# sourceMappingURL=comparator.d.ts.map